"""
The Complex Document is a Document that consists of several files.
It is possible to upload such a document, and to update from a file,
but not to edit. A complex document may contain several subobjects
holding the document content. However, it should be treated as an
document, not as a folder.
"""

ADD_CONTENT_PERMISSION = 'Add portal content'

import Globals, string, os, re, StringIO

from Globals import HTMLFile, HTML, InitializeClass
from Products.CMFDefault.Discussions import Discussable
from AccessControl import ClassSecurityInfo
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFDefault.utils import parseHeadersBody, SimpleHTMLParser, bodyfinder

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.WorkflowCore import WorkflowAction, afterCreate

from Products.ZipFolder.ZipImporter import ZipImporter
from Products.ZipFolder.HeadParser import HeadParser

from OFS.ObjectManager import ObjectManager
from Acquisition import aq_base

from ZPublisher.Converters import get_converter


filenamepattern=re.compile(r'[A-Za-z0-9][\w\_\-\.]*$')

# Factory type information -- makes ComplexDocument objects play nicely
# with the Types Tool (portal_types)
factory_type_information = (
    {'id': 'ComplexDocument',
     'content_icon': 'complexdocument_icon.png',
     'meta_type': 'Complex Document',
     'description': ('Complex Documents are documents that consist '
                     'of several files and can only be edited by uploading '
                     'Zip files containing the documents contents.'),
     'product': 'CMFComplexDocument',
     'factory': 'addComplexDocument',
     'immediate_view': 'metadata_edit_form',
     'allow_discussion': 'yes',
     'actions': ({'id': 'view',
                  'name': 'View',
                  'action': 'complexdocument_view',
                  'permissions': (CMFCorePermissions.View,)},
                 {'id': 'edit',
                  'name': 'Edit',
                  'action': 'complexdocument_edit_form',
                  'permissions': (CMFCorePermissions.ModifyPortalContent,)},
                 {'id': 'metadata',
                  'name': 'Metadata',
                  'action': 'metadata_edit_form',
                  'permissions': (CMFCorePermissions.ModifyPortalContent,)},
                 ),                     # End Actions
     },
    )



def addComplexDocument(self, id,
                       title='', description='',
                       file=''):
    """
    Add a complex Document
    """
    o=ComplexDocument(id, title, description,file)
    self._setObject(id,o)
    
    afterCreate(self.this()._getOb(id))

    
class ComplexDocument (ObjectManager,
                       PortalContent,
                       DefaultDublinCoreImpl,
                       ZipImporter):

    meta_type = 'Complex Document'
    effective_date = expiration_date = None
    _isDiscussable = 1

    _allowed_properties = ['start']

    security = ClassSecurityInfo()
    security.declareProtected(CMFCorePermissions.View,
                              'Description',
                              'folder_contents','this','isEffective',
                              'bobobase_modification_time')

    ## => bobobase_modification_time et al: Has to be explicit here?!

    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                              'edit')


    isPrincipiaFolderish=0
    

    def __init__(self, id, title='', description='', file=''):
        DefaultDublinCoreImpl.__init__(self)
        self.id=id
        self.title=title
        self.description=description
        self.edit(file)


    def edit(self, file=''):
        """
        Edit the Document
        """
        headers = {}
        if file and (type(file) is not type('')):

            removelist=self.objectIds(['File','Image','DTML Document'])
            self.manage_delObjects(ids=removelist)

            self.import_zipfile(file)
            self.do_cooking()


        
##         self.editMetadata(title=headers['Title'],
##                           subject=headers['Subject'],
##                           description=headers['Description'],
##                           contributors=headers['Contributors'],
##                           effective_date=headers['Effective_date'],
##                           expiration_date=headers['Expiration_date'],
##                           format=headers['Format'],
##                           language=headers['Language'],
##                           rights=headers['Rights'],
##                           )

    def add_file(self,path,basename,sf):
        
        if not filenamepattern.match(basename):
            return # skip ugly files
        
        suffix=os.path.splitext(basename)[1]

        folder=self

        if folder:
            if string.lower(suffix) in ['.htm', '.html']:
                # this is ugly
                sfx=StringIO.StringIO(self._edit_html(sf.getvalue()))
                sf.close()
                folder.manage_addDTMLDocument(id=basename, title='', file=sfx)

            elif string.lower(suffix) in ['.gif', '.jpg', '.jpeg', '.png']:
                folder.manage_addImage(id=basename, title='', file=sf)

            elif len(suffix)==0 and basename[-5:] == "_html":
                folder.manage_addDTMLDocument(id=basename, title='', file=sf)
                
            else:
                self._add_other_file(folder,path,basename,sf)


    def _edit_html(self, data):
        # a hook for filters to edit the imported html files
        hp=HeadParserLF()
        hp.feed(data)
        hp.close()
        
        return hp.accumulator 

    def _add_other_file(self,folder,path,basename,sf):
        # Hook for other known file types, default: add file objects
        if string.lower(basename) in ['props']:
            self._import_properties(sf)
        else:
            folder.manage_addFile(id=basename, title='', file=sf)
    
    def do_cooking(self):
        # maybe useful in the future
        pass

    def _import_properties(self,file):

        lines = file.readlines()

        for line in lines:
            # from FSPropertiesObject...
            propname, proptv = string.split( line, ':', 1)
            #XXX multi-line properties?
            proptype, propvstr = string.split( proptv, '=', 1)
            propname = string.strip(propname)
            proptv = string.strip(proptv)
            propvstr = string.strip(propvstr)
            converter = get_converter( proptype, lambda x: x )
            propvalue = converter( string.strip( propvstr ) )
            self._set_import_property(propname, proptype, propvalue)
            

    def _set_import_property(self,propname, proptype, propvalue):

        if propname in self._allowed_properties:
            if self.hasProperty(propname):
                self.manage_changeProperties({propname: propvalue})
            else:
                self.manage_addProperty(id=propname,value=propvalue,type=proptype)

            

    security.declareProtected(CMFCorePermissions.View,
                              'has_page','get_page','start_page')

    def has_page(self,id):
        return id in self.objectIds(['DTML Document'])

    def get_page(self,id):
        return self[id]
        
    def start_page(self):
        if self.hasProperty('start'):
            return self.getProperty('start')

        return 'index.html'


    def SearchableText(self):
        "text for indexing"
        text="%s %s" % (self.title, self.description)
        for x in self.objectValues(['DTML Document']):
            text=text+" "+str(x)
        return text

    edit = WorkflowAction(edit)
    folder_contents = PortalContent.index_html


    def manage_afterAdd(self, item, container):
        "call ancestors"
        ObjectManager.manage_afterAdd(self,item,container)
        PortalContent.manage_afterAdd(self,item,container)
            
    def manage_beforeDelete(self, item, container):
        "call ancestors"
        ObjectManager.manage_beforeDelete(self,item,container)
        PortalContent.manage_beforeDelete(self,item,container)
    

lf=re.compile("^[^/]*\.html?(\#.*)?$")

class HeadParserLF(HeadParser):
    # class parser + link filter (!)
    
    def start_a(self,attr):

        attrmap={}
        for (x,y) in attr:
            attrmap[x]=y

        if attrmap.has_key('href'):
            href=attrmap['href']
            if lf.match(href):
                attrmap['href']="&dtml-URL1;/view?page=%s" % href
                attr=[]
                for k in attrmap.keys():
                    attr.append((k,attrmap[k]))
                    
        self.unknown_starttag('a',attr)
        

InitializeClass(ComplexDocument)










